

import os
import multiprocessing

from pandas import Timestamp
import matplotlib as mpl
mpl.use('agg')
import matplotlib.pyplot as plt
import numpy as np

import dplume
import emiss.gauss

"""
Estimate CO2 emissions of Berlin using plumes detected by Monte-Carlo
experiment. Only use nsat=6 scenario.
"""



# timestamps for which plume detection for JAE has no multiple detections and
# at least 10 pixels for all instrument scenarios
jae_shared = {
    Timestamp('2015-01-13 11:00:00'),
    Timestamp('2015-03-16 11:00:00'),
    Timestamp('2015-03-22 11:00:00'),
    Timestamp('2015-04-15 11:00:00'),
    Timestamp('2015-04-20 11:00:00'),
    Timestamp('2015-04-21 11:00:00'),
    Timestamp('2015-04-23 11:00:00'),
    Timestamp('2015-06-11 11:00:00'),
    Timestamp('2015-07-01 11:00:00'),
    Timestamp('2015-07-04 11:00:00'),
    Timestamp('2015-08-12 11:00:00'),
    Timestamp('2015-08-22 11:00:00'),
    Timestamp('2015-09-19 11:00:00'),
    Timestamp('2015-09-29 11:00:00'),
    Timestamp('2015-10-11 11:00:00'),
    Timestamp('2015-10-12 11:00:00')
}

def do_mass_balance(filename, tracer, ssp, origin, co2_ssp, save_results=True,
                   show=False, degree=None):

    print(os.path.basename(filename))

    out_path = '/project/kug/smartcarb/results'
    if origin == 'Berlin':
        out_path = os.path.join(out_path, origin,
                                'flux_inversion/mass-balance/results_mc')
    else:
        out_path = os.path.join(out_path, origin,
                                'flux_inversion/mass-balance/')

    out_fname = os.path.join(
        out_path,
        '%s-%.1f' % (tracer, co2_ssp),
        os.path.splitext(os.path.basename(filename))[0]
    )

    if origin == 'Berlin':

        # wind
        wind_use_mean = True
        wind_hmin = 0.0
        wind_hmax = 500.0

        # curve
        if degree is None:
            degree = 2
        dmin = 15e3    # m
        dmax = np.inf

        is_mce = True

    else:
        # wind
        wind_use_mean = True
        wind_hmin = 150.0
        wind_hmax = 500.0

        # curve
        if degree is None:
            degree = 3
        dmin = 1e3    # m
        dmax = np.inf

        is_mce = False


    d,r,c = emiss.gauss.gauss_1d(
        filename, origin=origin, show=show,
        constant_emissions=False, load_results=False,
        save_results=save_results, out_filename=out_fname,
        load_plume=True,

        co2_ssp = co2_ssp,
        detection_tracer=tracer,

        # plume
        min_pixels=-1, # use all

        # curve
        degree=degree,
        dmin=dmin, dmax=dmax,
        force_origin=True,

        # background
        bg_area=10,       # 10x10 neighborhood

        # wind speed
        wind_use_mean=wind_use_mean, #
        wind_hmin=wind_hmin,        # Berlin: 0.0 m, Janschwalde: > 120 m
        wind_hmax=wind_hmax,         #       500.0 m,             < 3000 m

        is_mce=is_mce,
    )
    #plt.close("all")

    return d,r,c




def get_filenames(tracer, ssp, nsat=6, origin='Berlin', use_shared=False):

    # read results from plume detection
    ns = 37 if origin == 'Berlin' else 5
    ts = dplume.io.read_results(tracer, ssp, ns=ns, q=0.99, nsat=nsat,
                                origin=origin, use_mc=(origin=='Berlin'))

    # remove detected plumes with small number of pixels
    # or many misdetections
    if origin == 'Berlin':
        ts = ts[(ts.CO2_TP >= 100.0) & (ts.CO2_PPV >= 0.80)]
    else:
        ts = ts[(ts.CO2_pixels >= 10.0) & (ts.multiple_detections == 0)]

    if origin == 'Janschwalde' and use_shared:
        return ts.filename[jae_shared]
    else:
        return ts.filename


def main(tracer='CO2', ssp=0.5, origin='Berlin', co2_ssp=None, nsat=6, ncpu=24):

    use_shared =  origin == 'Janschwalde'

    if tracer == 'CO2':
        co2_ssp = ssp

    # read results from plume detection
    filenames = get_filenames(tracer, ssp, nsat, origin, use_shared=use_shared)

    print('number of files:', len(filenames))

    args = [(fn, tracer, ssp, origin, co2_ssp) for fn in filenames]

    if ncpu > 0:
        with multiprocessing.Pool(ncpu) as pool:
            pool.starmap(do_mass_balance, args)

        # collect results
        out_path = '/project/kug/smartcarb/results'
        if origin == 'Berlin':
            out_path = os.path.join(out_path, origin,
                                    'flux_inversion/mass-balance/results_mc')
        else:
            out_path = os.path.join(out_path, origin,
                                    'flux_inversion/mass-balance/')

        path = os.path.join(out_path, '%s-%.1f' % (tracer, co2_ssp))
        frame = emiss.gauss.results2timeseries(path)

        filename = os.path.join(out_path, '%s-%.1fppm_%s.dat' % (tracer, co2_ssp,
                                                              origin))
        frame.to_csv(filename, sep='\t')



if __name__ == '__main__':

    import emiss.gauss

    ncpu = 12

    co2_ssp = 'low'
    no2_ssp = 'none'

    for no2_ssp in ['none', 'high']:

        for co2_ssp in ['low', 'medium', 'high']:

            #emiss.gauss.main_prepare(start=0, ncpu=ncpu, overwrite=False,
            #                         co2_ssp=co2_ssp, no2_ssp=no2_ssp)

            #emiss.gauss.main_estimate('CO2', start=0, ncpu=ncpu, method='sub-areas',
            #                          co2_ssp=co2_ssp, no2_ssp=no2_ssp)

            if no2_ssp != 'none':
                emiss.gauss.main_estimate('CO2', start=0, ncpu=ncpu,
                                          method='two-gauss',
                                          co2_ssp=co2_ssp, no2_ssp=no2_ssp)

